package de.fraunhofer.iem.dataprovider.kpi.entity

import de.fraunhofer.iem.dataprovider.kpi.dto.KPIChildResponseDto
import de.fraunhofer.iem.dataprovider.kpi.dto.KPIResponseDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import jakarta.persistence.*
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*

@Entity
@Table(name = "kpi")
class KPIEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "repository_id")
    var repository: RepositoryEntity? = null

    @OneToMany(mappedBy = "from", cascade = [CascadeType.MERGE, CascadeType.REMOVE], orphanRemoval = true)
    var hierarchyEdges: MutableSet<KPIHierarchyEdgeEntitiy> = mutableSetOf()

    @Column(name = "value")
    var value: Int? = null

    @Column(name = "is_root")
    var isRoot: Boolean? = null

    @Column(name = "name")
    var name: String? = null

    @Column(name = "description", columnDefinition = "TEXT")
    var description: String? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "timestamp")
    var timestamp: Timestamp? = null

    fun toResultDto(): KPIResponseDto {
        val childrenList = mutableListOf<KPIChildResponseDto>()
        for (edge in this.hierarchyEdges) {
            val id = edge.to!!.id!!
            val weight = edge.weight!!
            childrenList.add(KPIChildResponseDto(id, weight))
        }
        return KPIResponseDto(
            this.id!!,
            this.value!!,
            this.name!!,
            this.description!!,
            this.isRoot!!,
            childrenList,
            this.repository!!.id!!
        )
    }
}