package de.fraunhofer.iem.dataprovider.repository.controller

import de.fraunhofer.iem.dataprovider.debug.*
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.util.*

@RestController
@RequestMapping("/repository")
class RepositoryController(private val repositoryService: RepositoryService) {

    private val logger = getLogger(javaClass)

    @GetMapping()
    suspend fun getAllRepositories(): List<RepositoryDto> {
        logger.info("Get all repositories")
        return repositoryService.getAllRepositories()
            .map { repositoryEntity: RepositoryEntity -> RepositoryDto(repositoryEntity.id!!, repositoryEntity.name!!) }
    }

    @GetMapping("/{id}")
    suspend fun getRepositoryById(@PathVariable id: UUID): RepositoryDto {
        logger.info("Get repository with id $id")
        val repositoryEntity = this.repositoryService.findRepoByID(id) ?: throw Exception("Repository can not be found")
        return RepositoryDto(repositoryEntity.id!!, repositoryEntity.name!!)
    }
}