package de.fraunhofer.iem.dataprovider.taskManager.tasks.dataQuery

import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.events.GitCloneDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.tasks.Task
import org.eclipse.jgit.api.Git
import java.nio.file.Paths
import java.util.*


data class GitRepository(val name: String, val uri: String, val id: Long, val platform: PlatformEnum) {
    fun toDbObject(): RepositoryEntity {
        val repo = RepositoryEntity()
        repo.name = this.name
        repo.repoId = this.id
        repo.platform = this.platform
        repo.url = this.uri
        return repo
    }
}

class CloneGitTask(
    private val gitRepository: GitRepository,
    private val repoDbId: UUID,
    override val responseChannel: suspend (task: Event) -> Unit,
    private val outputPath: String,
) : Task() {

    override suspend fun execute() {

        val outputDirName = "${gitRepository.name}-${taskID}"
        val outputDirectory = Paths.get(outputPath, outputDirName)
        logger.info("Cloning ${gitRepository.name} into $outputDirectory")

        val git: Git = Git.cloneRepository()
            .setURI(gitRepository.uri)
            .setDirectory(outputDirectory.toFile())
            .call()
        git.close()

        responseChannel(GitCloneDoneEvent(taskID, repoDbId, outputDirectory.toString()))

        logger.info("Finished cloning ${gitRepository.name}")
    }
}