package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.ort

import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.ort.model.AdvisorResultJson
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.ort.model.OrtJson
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.*
import java.io.IOException
import java.nio.file.Path

class OrtTask {

}

fun getHighestScore(results: Array<AdvisorResultJson>): Double {
    var highestScore = 0.0
    results.forEach { res ->
        res.vulnerabilities?.forEach { vul ->
            vul.references?.forEach { ref ->
                if (ref != null) {
                    if (ref.scoringSystem?.contains("cvss") == true) {
                        val score = ref.severity?.toDouble() ?: 0.0
                        if (highestScore < score) {
                            highestScore = score
                        }
                    }
                }
            }
        }
    }
    return highestScore
}

fun getOrtResultsFromFile(resultPath: Path): Array<AdvisorResultJson> {
    val resFile = resultPath.toFile()

    if (resFile.exists()) {
        val results = mutableListOf<AdvisorResultJson>()
        val resString = resFile.readText()
        val json = Json { ignoreUnknownKeys = true }

        val rawJson = json.decodeFromString<OrtJson>(resString)
        rawJson.advisor?.results?.advisorResults?.forEach { (key, value) ->
            if (value is JsonArray) {
                value.forEach { res ->
                    if (res is JsonObject) {

                        val flat = buildJsonObject {
                            res.entries.forEach { (eKey, eValue) ->
                                put(eKey, eValue)
                            }
                            put("identifier", key)
                        }

                        val advisorResult = json.decodeFromJsonElement<AdvisorResultJson>(flat)
                        results.add(advisorResult)
                    }
                }
            }
        }
        return results.toTypedArray()
    }

    throw IOException("File not found / Path is no file.")
}