package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif

import de.fraunhofer.iem.dataprovider.taskManager.events.SarifProcessDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.events.SarifProcessGroupDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.ProcessTask
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.SarifJson
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.getSarifFromFilePath
import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
import java.io.IOException
import java.nio.file.Path
import java.util.*

abstract class SarifTask : ProcessTask() {
    protected abstract val repoId: UUID
    protected abstract val toolRunService: ToolRunService
    protected abstract val resultPath: Path
    protected open val getSarif = ::getSarifFromFilePath

    override suspend fun handleProcessReturn(p: Process) {

        // TODO: check process exit codes. Current problem, if an analysis
        //  has findings it might return an exit code != 0 even tho the process finished correctly
        logger.info("Handle Process return in $javaClass")
        try {
            println(String(p.inputStream.readAllBytes()))
            println(String(p.errorStream.readAllBytes()))
            val sarifResult = getSarif(resultPath)
            toolRunService.createToolRunFromSarif(sarifResult, repoId)
            sendResult(sarifResult)
        } catch (e: IOException) {
            // TODO: send meaningful event and react on error
        }
    }

    override suspend fun sendResult(sarifJson: SarifJson) {

        val event = if (groupID != null) {
            SarifProcessGroupDoneEvent(this.taskID, repoId, groupID!!, sarifJson)
        } else {
            SarifProcessDoneEvent(this.taskID, repoId, sarifJson)
        }

        responseChannel(event)
    }
}
