package de.fraunhofer.iem.dataprovider.tool.service

import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.SarifRuleJson
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.asDbObject
import de.fraunhofer.iem.dataprovider.tool.entity.RuleEntity
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.tool.repository.RuleRepository
import org.springframework.stereotype.Service
import java.util.*

@Service
class RuleService(
    private val ruleRepository: RuleRepository,
) {

    /**
     * Returns a list of rules belonging to the given tool id.
     * Returns an empty list if id == null
     */
    fun getRulesByToolId(id: UUID?): List<RuleEntity> {
        return if (id != null) {
            ruleRepository.findByTool_Id(id)
        } else {
            listOf()
        }
    }

    fun createNewRulesAndSaveToTool(
        sarifRuleJsons: List<SarifRuleJson>,
        ruleEntities: MutableList<RuleEntity>,
        toolEntity: ToolEntity
    ) {
        sarifRuleJsons.forEach { sarifRule ->
            if (ruleEntities.find { it.sarifRuleId == sarifRule.id } == null) {
                val rule = ruleRepository.save(sarifRule.asDbObject())
                ruleEntities.add(rule)
                toolEntity.addRule(rule)
            }
        }
    }
}