package de.fraunhofer.iem.dataprovider.toolRun.entity

import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.tool.entity.RuleEntity
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import jakarta.persistence.*
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*

@Entity
@Table(name = "tool_run")
class ToolRunEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @OneToOne(orphanRemoval = true)
    @JoinColumn(name = "tool_id")
    var tool: ToolEntity? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "time_stamp")
    var timeStamp: Timestamp? = null

    @ManyToOne(fetch = FetchType.EAGER)
    var repository: RepositoryEntity? = null

    @OneToMany(
        mappedBy = "toolRun",
        orphanRemoval = true
    )
    var rules: MutableSet<RuleEntity> = mutableSetOf()

    @OneToMany(mappedBy = "toolRun", cascade = [CascadeType.ALL], orphanRemoval = true)
    var findings: MutableSet<FindingEntity> = mutableSetOf()

    fun addFindings(findingEntities: Collection<FindingEntity>) {
        findingEntities.forEach {
            this.addFinding(it)
        }
    }

    fun addFinding(findingEntity: FindingEntity) {
        findings.add(findingEntity)
        findingEntity.toolRun = this
    }

    fun addRules(ruleEntities: Collection<RuleEntity>) {
        ruleEntities.forEach {
            this.addRule(it)
        }
    }

    fun addRule(ruleEntity: RuleEntity) {
        rules.add(ruleEntity)
        ruleEntity.toolRun = this
    }
}

